%---------- ORIGINAL QUERY ----------------------------------------------------------------------------------------------

% Q7(Z,W,D):-R5(X,Y,Z),R6(X1,Y,W),R7(Y,U,D)


%---------- STATISTICS --------------------------------------------------------------------------------------------------

% All      variables = {X,Y,Z,X1,W,U,D}
% Free     variables = {Z,W,D}
% Join     variables = {Y}
% Key      variables = {X,X1,Y}
% Idle     variables = {U}
% Relevant variables = {X,Y,Z,X1,W,D}
%
% Encoding version   = 5
% Solver required    = gringo


%---------- COMPUTATION OF THE SAFE ANSWER ------------------------------------------------------------------------------

primalSub(X,Y,Z,X1,W,D) :- r5(X,Y,Z),r6(X1,Y,W),r7(Y,U,D).

suspectComp(kR5(X), nkR5(V2,V3)) :- primalSub(X,Y,Z,X1,W,D), r5(X,V2,V3).
suspectComp(kR6(X1), nkR6(V2,V3)) :- primalSub(X,Y,Z,X1,W,D), r6(X1,V2,V3).
suspectComp(kR7(Y), nkR7(V3)) :- primalSub(X,Y,Z,X1,W,D), r7(Y,V2,V3).

conflictingComp(K) :- suspectComp(K,NK1), suspectComp(K,NK2), NK1 > NK2.

safeAns(Z,W,D) :- primalSub(X,Y,Z,X1,W,D), not conflictingComp(kR5(X)), not conflictingComp(kR6(X1)), not conflictingComp(kR7(Y)).


%---------- HYPERGRAPH CONSTRUCTION -------------------------------------------------------------------------------------

conflictingSub(sID(X,Y,Z,X1,W,D), ans(Z,W,D)) :- primalSub(X,Y,Z,X1,W,D), not safeAns(Z,W,D).

keyComp(kR5(X), Ans) :- conflictingSub(sID(X,Y,Z,X1,W,D), Ans).
keyComp(kR6(X1), Ans) :- conflictingSub(sID(X,Y,Z,X1,W,D), Ans).
keyComp(kR7(Y), Ans) :- conflictingSub(sID(X,Y,Z,X1,W,D), Ans).

inSub(aR5(X,Y,Z), sID(X,Y,Z,X1,W,D)) :- conflictingSub(sID(X,Y,Z,X1,W,D), _).
inSub(aR6(X1,Y,W), sID(X,Y,Z,X1,W,D)) :- conflictingSub(sID(X,Y,Z,X1,W,D), _).
inSub(aR7(Y,D), sID(X,Y,Z,X1,W,D)) :- conflictingSub(sID(X,Y,Z,X1,W,D), _).

inKeyComp(aR5(X,V2,V3), kR5(X)) :- keyComp(kR5(X), Ans), suspectComp(kR5(X), nkR5(V2,V3)).
inKeyComp(aR6(X1,V2,V3), kR6(X1)) :- keyComp(kR6(X1), Ans), suspectComp(kR6(X1), nkR6(V2,V3)).
inKeyComp(aR7(Y,V3), kR7(Y)) :- keyComp(kR7(Y), Ans), suspectComp(kR7(Y), nkR7(V3)).


%---------- PRUNING ---------------------------------------------------------------------------------------------------

redundantComp(K,Ans) :- keyComp(K, Ans), inKeyComp(A,K), #count{S : inSub(A,S), conflictingSub(S,Ans)} = 0.

redundantComp(K,Ans) :- keyComp(K, Ans), inKeyComp(A,K), X = #count{S : inSub(A,S), conflictingSub(S,Ans)},
	                    #count{S : inSub(A,S), unfoundedSub(S,Ans)} >= X.

unfoundedSub(S,Ans) :- conflictingSub(S,Ans), inSub(A,S), inKeyComp(A,K), redundantComp(K,Ans).

residualSub(S,Ans) :- conflictingSub(S,Ans), not unfoundedSub(S,Ans).


%---------- FRAGMENTS IDENTIFICATION ----------------------------------------------------------------------------------

shareSub(K1,K2,Ans) :- 
	residualSub(S,Ans),
	inSub(A1,S), inSub(A2,S), A1 <> A2,
	inKeyComp(A1,K1), inKeyComp(A2,K2), K1 <> K2. 

ancestorOf(K1,K2,Ans) :- shareSub(K1,K2,Ans), K1 < K2.
ancestorOf(K1,K3,Ans) :- ancestorOf(K1,K2,Ans), shareSub(K2,K3,Ans), K1 < K3.

child(K,Ans) :- ancestorOf(_,K,Ans).

keyCompInFragment(K1, fID(K1,Ans)) :- ancestorOf(K1,K2,Ans), not child(K1,Ans).
keyCompInFragment(K2, fID(K1,Ans)) :- ancestorOf(K1,K2,Ans), not child(K1,Ans).

subInFragment(S,fID(KF,Ans)) :- residualSub(S,Ans), inSub(A,S), inKeyComp(A,K), keyCompInFragment(K,fID(KF,Ans)).

fragment(fID(K,Ans),Ans) :- keyCompInFragment(_,fID(K,Ans)).

atLeastOneFragment :- fragment(_,_).


%---------- COMPUTATION OF THE CONSISTENT ANSWER ----------------------------------------------------------------------

1 <= {activeFragment(F):fragment(F,Ans)} <= 1 :- atLeastOneFragment.

1 <= {activeAtom(A):inKeyComp(A,K)} <= 1 :- activeFragment(F), keyCompInFragment(K,F).

idleSub(S) :- activeFragment(F), subInFragment(S,F), inSub(A,S), not activeAtom(A).


%---------- REWRITTEN QUERY ---------------------------------------------------------------------------------------------

q(F,Z,W,D) :- activeFragment(F), subInFragment(S,F), not idleSub(S), fragment(F,ans(Z,W,D)).
q(F,Z,W,D) :- fragment(F,ans(Z,W,D)), not activeFragment(F).
%q(na,Z,W,D) :- safeAns(Z,W,D).
q(ca,Z,W,D) :- safeAns(Z,W,D).

