task(T,U) :- computationalTask(T,U,P,E).
taskT(T)  :- task(T,U).

% bind each computational task onto one tile
% 1 <= { bind(T,R) : tile(R) } <= 1 :- task(T,U).
:- task(T,U), not 1 <= { bind(T,R) : tile(R) } <= 1.
:- bind(T,R), not taskT(T).
:- bind(T,R), not tile(R).

% routing
root(C,R) :- send(X,C), bind(X,R).
sink(C,R) :- receive(X,C), bind(X,R).
% 1 <= { reached(C,R,S) : edge(R,S) } <= 1 :- sink(C,S), not root(C,S).
:- sink(C,S), not root(C,S), not 1 <= { reached(C,R,S) : edge(R,S) } <= 1.
:- reached(C,R,S), not edge(R,S).
:- reached(C,R,S), not sink(C,S).
:- reached(C,R,S), root(C,S).
sink(C,R) :- sink(C,S), reached(C,R,S).
:- receive(X,C), root(C,R), not sink(C,R).

% optimization 1
omu(SU/ST) :- ST = #count{ T : tile(T) }, SU = #sum{ U,T : task(T,U) }.
umsd((1000-Mean)/10) :- omu(Mean).
domain(10,1000).
domain(S,U - DU) :- domain(S+1,U), S > 0, umsd(DU).
:- not max_utilization(10,1000).
:- max_utilization(S,X), not domain(S,X).
:- max_utilization(S,X), S < 9, umsd(DU), not max_utilization(S+1,X+DU).
:- tile(R), max_utilization(S,X), X < #sum { U,T : bind(T,R), task(T,U) }.
% #maximize{1@3,S : max_utilization(S,_)}.
:~ domain(S,X), not max_utilization(S,X). [1@3,S]
:~ . [0@3]

% optimization 2
% #minimize{1@2,C:reached(C,R,S)}.
:~ reached(C,R,S). [1@2,C]
:~ . [0@2]

% optimization 3
% #minimize{1@1,reached(C,R,S):reached(C,R,S)}.
:~ reached(C,R,S). [1@1,C,R,S]
:~ . [0@1]

% #show bind/2.
% #show reached/3.
